require 'spec_helper'

module OpenWeather
  describe QueryBuilder do

    def build_query(*args)
      options = args.extract_options!
      use_id = options.delete(:use_id) || true
      args << options
      QueryBuilder.new(args, use_id).build
    end

    it "generates a coordinates query when lat and lng are given" do
      build_query(1, 2).should == { lat: 1, lon: 2 }
    end

    it "generates a coordinates query when object with to_coordinates method is given" do
      waypoint = OpenStruct.new(to_coordinates: [1, 2])
      build_query(waypoint).should == { lat: 1, lon: 2 }
    end

    it "generates a coordinates query when object with :lat, :lng method available" do
      waypoint = OpenStruct.new(lat: 1, lon: 2)
      build_query(waypoint).should == { lat: 1, lon: 2 }
    end

    it "generates a city query when an string is given" do
      build_query("Madrid").should == { q: "Madrid" }
    end

    it "generates an id query when a numeric string is given" do
      build_query("123").should == { id: "123" }
    end

    it "generates an id query when a numeric id is given" do
      build_query(123).should == { id: 123 }
    end

    it "parses internal attributes" do
      team = OpenStruct.new(location: "Madrid")
      build_query(team).should == { q: "Madrid" }
    end

    it "raise query error if location is nil" do
      team = OpenStruct.new(location: nil)
      expect {
        build_query(team)
      }.to raise_error(QueryError, 'you should provide at least one location')
    end

    it { build_query({lat: 1, lng: 2}, use_id: true).should == { lat: 1, lon: 2 } }
    it { build_query([1, 2]).should == { lat: 1, lon: 2 } }
    it { build_query([1, 2, 3]).should == { lat: 1, lon: 2 } }
    it { build_query(1, 2, 3).should == { lat: 1, lon: 2 } }

    it "raise query error if no arguments are given" do
      expect {
        build_query
      }.to raise_error(OpenWeather::QueryError, 'you should provide at least one location')
    end

    it "raise query error if can't figure out the query" do
      ::School = Class.new
      expect {
        build_query(School.new)
      }.to raise_error(QueryError, "Don't know how to obtain weather from School")
    end

  end
end
